/*======================================================================================

	Program:		Range Imaging Toolkit (RITK)
	Module:			ritkVisualizationWidget2D.cxx
	Version:		$Revision: 1761 $
	Date:			$Date: 2011-09-23 17:25:41 +0200 (Fr, 23 Sep 2011) $

	Copyright (c) 2010-2011
	Pattern Recognition Lab, 
    University Erlangen-Nuremberg, Germany
    All rights reserved.

	This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
	See RITKCopyright.txt for more information.

	http://www5.informatik.uni-erlangen.de/research/software/ritk

========================================================================================*/

#include "ritkVisualizationWidget2D.h"

#include <vtkImageViewer.h>
#include <vtkImageMapper.h>
#include <vtkPoints.h>
#include <vtkCellArray.h>
#include <vtkPointData.h>
#include <vtkUnsignedCharArray.h>
#include <vtkProperty.h>
#include <vtkLookupTable.h>
#include <vtkSphereSource.h>
#include <vtkActorCollection.h>

#include <vtkInteractorStyleImage.h>
#include <vtkInteractorStyleTrackballCamera.h>

#include <vtkBMPReader.h>
#include <vtkImageCast.h>

namespace ritk
{

VisualizationWidget2D::VisualizationWidget2D(QWidget *parent) :
	QWidget(parent)
{
	setupUi(this);

	// Register this smart pointer
	qRegisterMetaType<vtkSmartPointer<vtkImageActor> >("vtkSmartPointer<vtkImageActor>");

	// Initialize camera
	m_Camera = vtkSmartPointer<vtkCamera>::New();
	m_Camera->ComputeViewPlaneNormal();
	m_Camera->SetPosition(0,0,0.1);
	m_Camera->SetFocalPoint(0,0,0);
	m_Camera->SetClippingRange( 0.1, 1000.);
	m_Camera->SetViewUp(0,-1,0);
	m_Camera->Zoom(1.5);
	m_CameraInitFlag = false; // ???

	// Set up render pipeline
	m_Renderer = vtkSmartPointer<vtkRenderer>::New();
	m_Renderer->SetActiveCamera(m_Camera);
	//m_Renderer->SetAmbient(0,0,0);
	//m_Renderer->LightFollowCameraOn();
	m_Renderer->SetBackground(0,0,0);
	
	m_Renderer->ResetCamera();
	m_RenderWindow = vtkSmartPointer<vtkRenderWindow>::New();
	m_RenderWindow->AddRenderer(m_Renderer);

	// The QVTK widget interactor
	m_InteractorStyleImage = vtkSmartPointer<vtkInteractorStyleImage>::New();
	m_Interactor = vtkSmartPointer<QVTKInteractor>::New();
	m_Interactor->SetInteractorStyle(m_InteractorStyleImage);
	m_RenderWindow->SetInteractor(m_Interactor);
	m_Interactor->Initialize();
	
	m_VTKWidget->SetRenderWindow(m_RenderWindow);
}


VisualizationWidget2D::~VisualizationWidget2D()
{
}


//----------------------------------------------------------------------------
vtkRenderWindow*
VisualizationWidget2D::GetRenderWindow()
{
	return m_VTKWidget->GetRenderWindow();	
}


//----------------------------------------------------------------------------
void
VisualizationWidget2D::UpdateGUI()
{

	if ( !m_CameraInitFlag )
		m_Renderer->ResetCamera();

	// Initialize camera
	if ( !m_CameraInitFlag )
	{
		m_Camera->Zoom(1.5);
		m_CameraInitFlag = true;
	}
	
	m_VTKWidget->update();	
}


//----------------------------------------------------------------------------
void
VisualizationWidget2D::AddActor(vtkSmartPointer<vtkImageActor> Actor)
{
	this->m_VTKWidget->Lock();
	m_Renderer->AddViewProp(Actor);
	this->m_VTKWidget->Unlock();
	m_VTKWidget->update();
}


//----------------------------------------------------------------------------
void
VisualizationWidget2D::RemoveActor(vtkSmartPointer<vtkImageActor> Actor)
{
	this->m_VTKWidget->Lock();
	m_Renderer->RemoveViewProp(Actor);	
	this->m_VTKWidget->Unlock();
	m_VTKWidget->update();
}


//----------------------------------------------------------------------------
void
VisualizationWidget2D::AddProp3D(vtkSmartPointer<vtkProp3D> Prop3D)
{
	this->m_VTKWidget->Lock();
	m_Renderer->AddViewProp(Prop3D);
	this->m_VTKWidget->Unlock();
	m_VTKWidget->update();
}


//----------------------------------------------------------------------------
void
VisualizationWidget2D::RemoveProp3D(vtkSmartPointer<vtkProp3D> Prop3D)
{
	this->m_VTKWidget->Lock();
	m_Renderer->RemoveViewProp(Prop3D);	
	this->m_VTKWidget->Unlock();
	m_VTKWidget->update();
}



} // end namespace ritk
